<?php

namespace Chemical\Reaction\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Chemical\Reaction\Http\Requests\ReactionRequest;
use Chemical\Reaction\Interfaces\ReactionRepositoryInterface;
use Chemical\Reaction\Models\Reaction;

/**
 * Resource controller class for reaction.
 */
class ReactionResourceController extends BaseController
{

    /**
     * Initialize reaction resource controller.
     *
     * @param type ReactionRepositoryInterface $reaction
     *
     * @return null
     */
    public function __construct(ReactionRepositoryInterface $reaction)
    {
        parent::__construct();
        $this->repository = $reaction;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Chemical\Reaction\Repositories\Criteria\ReactionResourceCriteria::class);
    }

    /**
     * Display a list of reaction.
     *
     * @return Response
     */
    public function index(ReactionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Chemical\Reaction\Repositories\Presenter\ReactionPresenter::class)
                ->$function();
        }

        $reactions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('reaction::reaction.names'))
            ->view('reaction::reaction.index', true)
            ->data(compact('reactions', 'view'))
            ->output();
    }

    /**
     * Display reaction.
     *
     * @param Request $request
     * @param Model   $reaction
     *
     * @return Response
     */
    public function show(ReactionRequest $request, Reaction $reaction)
    {

        if ($reaction->exists) {
            $view = 'reaction::reaction.show';
        } else {
            $view = 'reaction::reaction.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('reaction::reaction.name'))
            ->data(compact('reaction'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new reaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReactionRequest $request)
    {

        $reaction = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('reaction::reaction.name')) 
            ->view('reaction::reaction.create', true) 
            ->data(compact('reaction'))
            ->output();
    }

    /**
     * Create new reaction.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReactionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $reaction                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('reaction::reaction.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('reaction/reaction/' . $reaction->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/reaction/reaction'))
                ->redirect();
        }

    }

    /**
     * Show reaction for editing.
     *
     * @param Request $request
     * @param Model   $reaction
     *
     * @return Response
     */
    public function edit(ReactionRequest $request, Reaction $reaction)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('reaction::reaction.name'))
            ->view('reaction::reaction.edit', true)
            ->data(compact('reaction'))
            ->output();
    }

    /**
     * Update the reaction.
     *
     * @param Request $request
     * @param Model   $reaction
     *
     * @return Response
     */
    public function update(ReactionRequest $request, Reaction $reaction)
    {
        try {
            $attributes = $request->all();

            $reaction->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('reaction::reaction.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('reaction/reaction/' . $reaction->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('reaction/reaction/' . $reaction->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the reaction.
     *
     * @param Model   $reaction
     *
     * @return Response
     */
    public function destroy(ReactionRequest $request, Reaction $reaction)
    {
        try {

            $reaction->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('reaction::reaction.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('reaction/reaction/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('reaction/reaction/' . $reaction->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple reaction.
     *
     * @param Model   $reaction
     *
     * @return Response
     */
    public function delete(ReactionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('reaction::reaction.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('reaction/reaction'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/reaction/reaction'))
                ->redirect();
        }

    }

    /**
     * Restore deleted reactions.
     *
     * @param Model   $reaction
     *
     * @return Response
     */
    public function restore(ReactionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('reaction::reaction.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/reaction/reaction'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/reaction/reaction/'))
                ->redirect();
        }

    }

}
